@@ -4,7 +4,6 @@ import unittest
 import os
 import glob
 import shutil
-import re
 
 import dripy
 import thornpy
@@ -168,16 +167,68 @@ class Test_DrillSim(unittest.TestCase):
         # Create a solver settings object
         self.solver_settings = adripy.DrillSolverSettings(TEST_SOLVER_SETTINGS_NAME)
 
+        # If the "existing" drill sims don't exit, create them
+        for directory in [TEST_EXISTING_DRILLSIM_DIRECTORY, TEST_EXISTING_UNBUILT_DRILLSIM_DIRECTORY]:
+            if not os.path.exists(directory):
+                create_existing_drillsim(directory)
+
         drill_sim = DrillSim.read_from_directory(TEST_EXISTING_DRILLSIM_DIRECTORY)
         if drill_sim.built is False:
             drill_sim.build()
 
+    def test_modify_solver_settings_after_build(self):
+        # Load the drill sim
+        drill_sim = DrillSim.read_from_directory(TEST_EXISTING_DRILLSIM_DIRECTORY)
+        
+        # Backup the drill sim's acf file and ssf files
+        shutil.copyfile(os.path.join(drill_sim.directory, drill_sim.acf_filename), os.path.join(drill_sim.directory, drill_sim.acf_filename) + '.tmp')
+        shutil.copyfile(drill_sim.solver_settings.filename, drill_sim.solver_settings.filename + '.tmp')
+
+        # Read in the new solver settings
+        new_solver_settings = adripy.DrillSolverSettings.read_from_file(TEST_SOLVER_SETTINGS_FILE)
+        
+        # Update the drill sim with the new solver setings
+        drill_sim.modify_solver_settings(new_solver_settings)
+
+        failures = check_file_contents(os.path.join(drill_sim.directory, drill_sim.acf_filename), EXPECTED_DRILLSIM_ACF_TEXT_AFTER_SSF_CHANGE)
+
+        # Replace files
+        os.remove(os.path.join(drill_sim.directory, drill_sim.acf_filename))
+        os.rename(os.path.join(drill_sim.directory, drill_sim.acf_filename) + '.tmp', os.path.join(drill_sim.directory, drill_sim.acf_filename))
+        os.remove(drill_sim.solver_settings.filename)
+        os.rename(drill_sim.solver_settings.filename + '.tmp', drill_sim.solver_settings.filename)
+
+        # Assert no failures
+        self.assertListEqual(failures, [])
+
+    def test_modify_solver_settings_before_build(self):
+        # Load the drill sim
+        drill_sim = DrillSim.read_from_directory(TEST_EXISTING_UNBUILT_DRILLSIM_DIRECTORY)
+        
+        # Backup the drill sim's acf file and ssf files
+        shutil.copyfile(drill_sim.solver_settings.filename, drill_sim.solver_settings.filename + '.tmp')
+
+        # Read in the new solver settings
+        new_solver_settings = adripy.DrillSolverSettings.read_from_file(TEST_SOLVER_SETTINGS_FILE)
+        
+        # Update the drill sim with the new solver setings
+        drill_sim.modify_solver_settings(new_solver_settings)
+
+        failures = check_file_contents(drill_sim.solver_settings.filename , EXPECTED_DRILLSIM_SSF_TEXT_AFTER_SSF_CHANGE)
+
+        # Replace files
+        os.remove(drill_sim.solver_settings.filename)
+        os.rename(drill_sim.solver_settings.filename + '.tmp', drill_sim.solver_settings.filename)
+
+        # Assert no failures
+        self.assertListEqual(failures, [])
+
     def test_drillsim_from_existing_event_file_2018(self):
         event_from_file = adripy.event.DrillEvent.read_from_file(TEST_EVENT_FILE)
         drill_sim = DrillSim(self.drill_string, event_from_file, self.solver_settings, TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME)
         drill_sim.build()
 
-        failures = check_file_contents(drill_sim.acf_filename, EXPECTED_DRILLSIM_FROM_EXISTING_EVENT_FILE_ACF_TEXT)
+        failures = check_file_contents(os.path.join(drill_sim.directory, drill_sim.acf_filename), EXPECTED_DRILLSIM_FROM_EXISTING_EVENT_FILE_ACF_TEXT)
         self.assertListEqual(failures, [])
 
     def test_drillsim_from_existing_event_file_2019(self):
@@ -185,7 +236,7 @@ class Test_DrillSim(unittest.TestCase):
         drill_sim = DrillSim(self.drill_string, event_from_file, self.solver_settings, TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME)
         drill_sim.build()
 
-        failures = check_file_contents(drill_sim.acf_filename, EXPECTED_DRILLSIM_FROM_EXISTING_EVENT_FILE_ACF_TEXT)
+        failures = check_file_contents(os.path.join(drill_sim.directory, drill_sim.acf_filename), EXPECTED_DRILLSIM_FROM_EXISTING_EVENT_FILE_ACF_TEXT)
         self.assertListEqual(failures, [])
 
     def test_string_file_contents(self):
@@ -194,7 +245,7 @@ class Test_DrillSim(unittest.TestCase):
         """
         drill_sim = DrillSim(self.drill_string, self.event, self.solver_settings, TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME)
 
-        failures = check_file_contents(drill_sim.string_filename, EXPECTED_DRILLSIM_STRING_FILE_TEXT)
+        failures = check_file_contents(os.path.join(drill_sim.directory, drill_sim.string_filename), EXPECTED_DRILLSIM_STRING_FILE_TEXT)
 
         self.assertListEqual(failures, [])
         
@@ -239,7 +290,7 @@ class Test_DrillSim(unittest.TestCase):
         drill_sim.build()
 
         # Assert that the acf file exists
-        self.assertTrue(os.path.exists(drill_sim.acf_filename))
+        self.assertTrue(os.path.exists(os.path.join(drill_sim.directory, drill_sim.acf_filename)))
 
     def test_build_evt_contents(self):
         """Tests that the event file created in the DrillSim directory has the correct contents.        
@@ -261,7 +312,7 @@ class Test_DrillSim(unittest.TestCase):
         drill_sim.build()
         adm_file = os.path.join(TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME + '.adm')
         n_adm_lines = len(open(adm_file,'r').readlines())
-        expected_n_adm_lines = 12457
+        expected_n_adm_lines = 12538
         self.assertEqual(n_adm_lines, expected_n_adm_lines)
 
     def test_write_tiem_orbit_files_event_filename(self):
@@ -283,7 +334,7 @@ class Test_DrillSim(unittest.TestCase):
 
         # Test the filename
         expected_filename = os.path.join(TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME + '.str')        
-        actual_filename = drill_sim.string_filename        
+        actual_filename = os.path.join(drill_sim.directory, drill_sim.string_filename)
         self.assertEqual(actual_filename, expected_filename)
 
     def test_write_tiem_orbit_files_ssf_filename(self):
@@ -305,7 +356,7 @@ class Test_DrillSim(unittest.TestCase):
 
         expected_contents = [
             drill_sim.event.filename,
-            drill_sim.string_filename,
+            os.path.join(drill_sim.directory, drill_sim.string_filename),
             drill_sim.solver_settings.filename
         ]
         for tool in drill_sim.string.tools:
@@ -323,7 +374,7 @@ class Test_DrillSim(unittest.TestCase):
         """
         drill_sim = DrillSim(self.drill_string, self.event, self.solver_settings, TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME)
         
-        hole_filepath = adripy.get_TO_param(drill_sim.string_filename, 'Hole_Property_File')
+        hole_filepath = adripy.get_TO_param(os.path.join(drill_sim.directory, drill_sim.string_filename), 'Hole_Property_File')
         self.assertFalse(thornpy.utilities.convert_path(drill_sim.directory) in thornpy.utilities.convert_path(hole_filepath))
 
     def test_relativity_in_string_event_reference(self):
@@ -331,7 +382,7 @@ class Test_DrillSim(unittest.TestCase):
         """
         drill_sim = DrillSim(self.drill_string, self.event, self.solver_settings, TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME)
         
-        event_filepath = adripy.get_TO_param(drill_sim.string_filename, 'Event_Property_File')
+        event_filepath = adripy.get_TO_param(os.path.join(drill_sim.directory, drill_sim.string_filename), 'Event_Property_File')
         self.assertFalse(thornpy.utilities.convert_path(drill_sim.directory) in thornpy.utilities.convert_path(event_filepath))
 
     def test_relativity_in_string_pdc_reference(self):
@@ -339,7 +390,7 @@ class Test_DrillSim(unittest.TestCase):
         """
         drill_sim = DrillSim(self.drill_string, self.event, self.solver_settings, TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME)
         
-        _name, pdc_filepath, _so, _gn = adripy.get_tool_name(drill_sim.string_filename, 'pdc_bit', return_full_path=True)
+        _name, pdc_filepath, _so, _gn = adripy.get_tool_name(os.path.join(drill_sim.directory, drill_sim.string_filename), 'pdc_bit', return_full_path=True)
 
         self.assertFalse(thornpy.utilities.convert_path(drill_sim.directory) in thornpy.utilities.convert_path(pdc_filepath))
 
@@ -351,7 +402,7 @@ class Test_DrillSim(unittest.TestCase):
         drill_sim.build()
         expected_contents = [
             drill_sim.event.filename,
-            drill_sim.string_filename,
+            os.path.join(drill_sim.directory, drill_sim.string_filename),
             drill_sim.solver_settings.filename,
             os.path.join(drill_sim.directory, drill_sim.adm_filename),
             os.path.join(drill_sim.directory, drill_sim.acf_filename),
@@ -492,6 +543,56 @@ class Test_DrillSim(unittest.TestCase):
         # Remove adm and acf in the unbuilt drillsim directory
         for file in glob.glob(os.path.join(TEST_EXISTING_UNBUILT_DRILLSIM_DIRECTORY, '*.acf')) + glob.glob(os.path.join(TEST_EXISTING_UNBUILT_DRILLSIM_DIRECTORY, '*.adm')):
             os.remove(file)
-            
+
+def create_existing_drillsim(directory):
+    # Create a test config file containing the test database
+    adripy.create_cfg_file(TEST_CONFIG_FILENAME, [TEST_DATABASE_PATH, TEST_NEW_DATABASE_PATH])
+
+    # Create a DrillTool object representing a stabilizer
+    pdc_bit = adripy.DrillTool(TEST_PDC_FILE)
+
+    # Create a DrillTool object representing a motor
+    motor = adripy.DrillTool(TEST_MOTOR_FILE)
+
+    # Create a DrillTool object representing a stabilizer
+    stabilizer = adripy.DrillTool(TEST_STABILIZER_FILE)
+
+    # Create a DrillTool object representing a drill pipe
+    drill_pipe = adripy.DrillTool(TEST_DRILLPIPE_FILE)
+
+    # Create a DrillTool object representing EUS
+    eus = adripy.DrillTool(TEST_EUS_FILE)
+
+    # Create a DrillTool object representing a top drive
+    top_drive = adripy.DrillTool(TEST_TOP_DRIVE_FILE)
+
+    # Create a DrillString object
+    drill_string = adripy.DrillString(TEST_STRING_NAME, TEST_EXISTING_HOLE_FILE, TEST_EVENT_FILE)
+
+    # Add the DrillTool objects to the DrillString object
+    drill_string.add_tool(pdc_bit, measure='yes')
+    drill_string.add_tool(motor, measure='yes')
+    drill_string.add_tool(stabilizer, measure='yes')
+    drill_string.add_tool(drill_pipe, joints=20, group_name='Upper_DP_Group')
+    drill_string.add_tool(eus, joints=20, group_name='equivalent_pipe', equivalent=True)
+    drill_string.add_tool(top_drive)
+
+    # Create an event object
+    event = adripy.DrillEvent(TEST_EVENT_NAME,2000, 3)
+    event.add_simulation_step(10)
+    event.add_simulation_step(100)
+    event.add_ramp('FLOW_RATE', 0, 15, 500)
+    event.add_ramp('ROTARY_RPM', 15, 15, 60)
+    event.add_ramp('WOB', 30, 15, 50000)
+    event.add_ramp('ROP', 30, 15, 100)
+
+    # Create a solver settings object
+    solver_settings = adripy.DrillSolverSettings(TEST_SOLVER_SETTINGS_NAME)
+
+    if os.path.exists(directory):
+        shutil.rmtree(directory, ignore_errors=True)
+
+    drill_sim = adripy.DrillSim(drill_string, event, solver_settings, directory, TEST_ANALYSIS_NAME)
+
 if __name__ == '__main__':
     unittest.main()
